@@ -752,10 +752,11 @@ let update_with_init_config ?(overwrite=false) config init_config =
     else conf
   in
   config |>
-  match I.jobs init_config with
+  (match I.jobs init_config with
       | Some j -> setifnew C.jobs C.with_jobs j
-      | None -> fun c -> c |>
+      | None -> fun c -> c) |>
   setifnew C.dl_tool C.with_dl_tool_opt (I.dl_tool init_config) |>
+  setifnew C.dl_cache C.with_dl_cache (I.dl_cache init_config) |>
   setifnew C.dl_jobs C.with_dl_jobs
     (OpamStd.Option.default OpamStateConfig.(default.dl_jobs)
        (I.dl_jobs init_config)) |>
@@ -1479,39 +1480,16 @@ module PIN = struct
   open OpamPinCommand
 
   let post_pin_action st was_pinned names =
-    let pkgs =
-      let newly = st.pinned -- was_pinned in
-      let old =
-        OpamPackage.packages_of_names was_pinned
-          OpamPackage.Name.Set.Op.(
-            OpamPackage.Name.Set.of_list names
-            -- OpamPackage.names_of_packages newly)
-      in
-      newly ++ old
-    in
-    let no_depexts =
-      not (OpamFile.Config.depext st.switch_global.config)
-      || OpamSysPkg.Set.is_empty
-        ((OpamPackage.Set.fold (fun pkg acc ->
-             OpamSysPkg.Set.union acc (OpamSwitchState.depexts st pkg)))
-           pkgs OpamSysPkg.Set.empty)
+    let names =
+      OpamPackage.Set.Op.(st.pinned -- was_pinned)
+      |> OpamPackage.names_of_packages
+      |> (fun s ->
+          List.fold_left
+            (fun s p -> OpamPackage.Name.Set.add p s)
+            s names)
+      |> OpamPackage.Name.Set.elements
     in
     try
-      let st =
-        if no_depexts then st else
-        let st =
-          { st with sys_packages =  lazy (
-                OpamPackage.Map.union (fun _ n -> n)
-                  (Lazy.force st.sys_packages)
-                  (OpamSwitchState.depexts_status_of_packages st pkgs)
-              )}
-        in
-        { st with available_packages = lazy (
-              OpamPackage.Set.filter (fun nv ->
-                  OpamSwitchState.depexts_unavailable st nv = None)
-                (Lazy.force st.available_packages)
-            )}
-      in
       upgrade_t
         ~strict_upgrade:false ~auto_install:true ~ask:true ~terse:true
         ~all:false
