@@ -157,10 +157,13 @@ module VCS : OpamVCS.VCS = struct
        match r.OpamProcess.r_stdout with
        | []      -> Done None
        | full::_ ->
-         if String.length full > 8 then
-           Done (Some (String.sub full 0 8))
-         else
-           Done (Some full))
+         Done (Some full))
+
+  let clean repo_root =
+    git repo_root [ "clean"; "-fdx" ]
+    @@> fun r ->
+    OpamSystem.raise_on_process_error r;
+    Done ()
 
   let reset_tree repo_root repo_url =
     let rref = remote_ref repo_url in
@@ -169,11 +172,7 @@ module VCS : OpamVCS.VCS = struct
     if OpamProcess.is_failure r then
       OpamSystem.internal_error "Git error: %s not found." rref
     else
-      git repo_root [ "clean"; "-fdx" ]
-      @@> fun r ->
-      if OpamProcess.is_failure r then
-        OpamSystem.internal_error "Git error: %s not found." rref
-      else
+      clean repo_root @@+ fun () ->
       if OpamFilename.exists (repo_root // ".gitmodules") then
         git repo_root [ "submodule"; "update"; "--init"; "--recursive" ]
         @@> fun r ->
